"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDownloadFilePlugins = void 0;

var _s = _interopRequireDefault(require("aws-sdk/clients/s3"));

var _sanitizeFilename = _interopRequireDefault(require("sanitize-filename"));

var _path = _interopRequireDefault(require("path"));

var _utils = require("../utils");

var _loaders = _interopRequireDefault(require("../transform/loaders"));

var _gateway = require("@webiny/handler-aws/gateway");

const MAX_RETURN_CONTENT_LENGTH = 5000000; // ~4.77MB

const DEFAULT_CACHE_MAX_AGE = 30758400; // 1 year

const PRESIGNED_URL_EXPIRATION = 900; // 15 minutes

/**
 * Based on given path, extracts file key and additional options sent via query params.
 */

const extractFilenameOptions = request => {
  const path = (0, _sanitizeFilename.default)(request.params.path);
  return {
    filename: decodeURI(path),
    options: request.query,
    extension: _path.default.extname(path)
  };
};

const getS3Object = async (request, s3, context) => {
  const {
    options,
    filename,
    extension
  } = extractFilenameOptions(request);
  const params = (0, _utils.getObjectParams)(filename);
  const objectHead = await s3.headObject(params).promise();
  const contentLength = objectHead.ContentLength ? objectHead.ContentLength : 0;

  for (const loader of _loaders.default) {
    const canProcess = loader.canProcess({
      context,
      s3,
      options,
      file: {
        name: filename,
        extension,
        contentLength
      }
    });

    if (!canProcess) {
      continue;
    }

    return loader.process({
      context,
      s3,
      options,
      file: {
        name: filename,
        extension,
        contentLength
      }
    });
  } // If no processors handled the file request, just return the S3 object taking its size into consideration.


  let object;

  if (contentLength < MAX_RETURN_CONTENT_LENGTH) {
    object = await s3.getObject(params).promise();
  }

  return {
    object,
    params
  };
};

const createDownloadFilePlugins = () => {
  return [new _gateway.RoutePlugin(({
    onGet,
    context
  }) => {
    onGet("/files/:path", async (request, reply) => {
      const {
        region
      } = (0, _utils.getEnvironment)();
      const s3 = new _s.default({
        region
      });
      const {
        params,
        object
      } = await getS3Object(request, s3, context); // If there's an "object", it means we can return its body directly.

      if (object) {
        return reply.headers({
          "Content-Type": object.ContentType,
          "Cache-Control": `public, max-age=${DEFAULT_CACHE_MAX_AGE}`,
          "x-webiny-base64-encoded": true
        }).send(object.Body || "");
      }

      const presignedUrl = s3.getSignedUrl("getObject", {
        Bucket: params.Bucket,
        Key: params.Key,
        Expires: PRESIGNED_URL_EXPIRATION
      }); // Lambda can return max 6MB of content, so if our object's size is larger, we are sending
      // a 301 Redirect, redirecting the user to the public URL of the object in S3.

      return reply.code(301).headers({
        Location: presignedUrl,
        "Cache-Control": "public, max-age=" + PRESIGNED_URL_EXPIRATION
      }).send("");
    });
  })];
};

exports.createDownloadFilePlugins = createDownloadFilePlugins;