"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _mdbid = _interopRequireDefault(require("mdbid"));

var _handlerGraphql = require("@webiny/handler-graphql");

var _apiSecurity = require("@webiny/api-security");

var _checkBasePermissions = _interopRequireDefault(require("./utils/checkBasePermissions"));

var _api = require("@webiny/api");

var _FilePlugin = require("../definitions/FilePlugin");

var _FilesStorageOperationsProviderPlugin = require("../definitions/FilesStorageOperationsProviderPlugin");

var _error = _interopRequireDefault(require("@webiny/error"));

var _lifecycleEvents = require("./utils/lifecycleEvents");

/**
 * Package mdbid does not have types.
 */
// @ts-ignore
const BATCH_CREATE_MAX_FILES = 20;
/**
 * If permission is limited to "own" files only, check that current identity owns the file.
 */

const checkOwnership = (file, permission, context) => {
  if ((permission === null || permission === void 0 ? void 0 : permission.own) === true) {
    const identity = context.security.getIdentity();

    if (file.createdBy.id !== identity.id) {
      throw new _apiSecurity.NotAuthorizedError();
    }
  }
};

const getLocaleCode = context => {
  if (!context.i18n) {
    throw new _error.default("Missing i18n on the FileManagerContext.", "MISSING_I18N");
  }

  const locale = context.i18n.getContentLocale();

  if (!locale) {
    throw new _error.default("Missing content locale on the FileManagerContext.", "MISSING_I18N_CONTENT_LOCALE");
  }

  if (!locale.code) {
    throw new _error.default("Missing content locale code on the FileManagerContext.", "MISSING_I18N_CONTENT_LOCALE_CODE");
  }

  return locale.code;
};

const filesContextCrudPlugin = new _api.ContextPlugin(async context => {
  const pluginType = _FilesStorageOperationsProviderPlugin.FilesStorageOperationsProviderPlugin.type;
  const providerPlugin = context.plugins.byType(pluginType).find(() => true);

  if (!providerPlugin) {
    throw new _error.default(`Missing "${pluginType}" plugin.`, "PLUGIN_NOT_FOUND", {
      type: pluginType
    });
  }

  const storageOperations = await providerPlugin.provide({
    context
  });

  if (!context.fileManager) {
    context.fileManager = {};
  }

  const filePlugins = context.plugins.byType(_FilePlugin.FilePlugin.type);
  context.fileManager.files = {
    async getFile(id) {
      const permission = await (0, _checkBasePermissions.default)(context, {
        rwd: "r"
      });
      const file = await storageOperations.get({
        where: {
          id,
          tenant: context.tenancy.getCurrentTenant().id,
          locale: getLocaleCode(context)
        }
      });

      if (!file) {
        throw new _handlerGraphql.NotFoundError(`File with id "${id}" does not exists.`);
      }

      checkOwnership(file, permission, context);
      return file;
    },

    async createFile(input) {
      await (0, _checkBasePermissions.default)(context, {
        rwd: "w"
      });
      const identity = context.security.getIdentity();
      const tenant = context.tenancy.getCurrentTenant();
      const id = (0, _mdbid.default)();
      const file = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, input), {}, {
        tags: Array.isArray(input.tags) ? input.tags : [],
        id,
        meta: (0, _objectSpread2.default)({
          private: false
        }, input.meta || {}),
        tenant: tenant.id,
        createdOn: new Date().toISOString(),
        createdBy: {
          id: identity.id,
          displayName: identity.displayName,
          type: identity.type
        },
        locale: getLocaleCode(context),
        webinyVersion: context.WEBINY_VERSION
      });

      try {
        await (0, _lifecycleEvents.runLifecycleEvent)("beforeCreate", {
          context,
          plugins: filePlugins,
          data: file
        });
        const result = await storageOperations.create({
          file
        });
        await (0, _lifecycleEvents.runLifecycleEvent)("afterCreate", {
          context,
          plugins: filePlugins,
          data: file,
          file: result
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not create a file.", ex.code || "CREATE_FILE_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          file
        }));
      }
    },

    async updateFile(id, input) {
      const permission = await (0, _checkBasePermissions.default)(context, {
        rwd: "w"
      });
      const original = await storageOperations.get({
        where: {
          id,
          tenant: context.tenancy.getCurrentTenant().id,
          locale: getLocaleCode(context)
        }
      });

      if (!original) {
        throw new _handlerGraphql.NotFoundError(`File with id "${id}" does not exists.`);
      }

      checkOwnership(original, permission, context);
      const file = (0, _objectSpread2.default)((0, _objectSpread2.default)((0, _objectSpread2.default)({}, original), input), {}, {
        tags: Array.isArray(input.tags) ? input.tags : Array.isArray(original.tags) ? original.tags : [],
        id: original.id,
        webinyVersion: context.WEBINY_VERSION
      });

      try {
        await (0, _lifecycleEvents.runLifecycleEvent)("beforeUpdate", {
          context,
          plugins: filePlugins,
          original,
          data: file
        });
        const result = await storageOperations.update({
          original,
          file
        });
        await (0, _lifecycleEvents.runLifecycleEvent)("afterUpdate", {
          context,
          plugins: filePlugins,
          original,
          data: file,
          file: result
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not update a file.", ex.code || "UPDATE_FILE_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          original,
          file
        }));
      }
    },

    async deleteFile(id) {
      const permission = await (0, _checkBasePermissions.default)(context, {
        rwd: "d"
      });
      const file = await storageOperations.get({
        where: {
          id,
          tenant: context.tenancy.getCurrentTenant().id,
          locale: getLocaleCode(context)
        }
      });

      if (!file) {
        throw new _handlerGraphql.NotFoundError(`File with id "${id}" does not exists.`);
      }

      checkOwnership(file, permission, context);

      try {
        await (0, _lifecycleEvents.runLifecycleEvent)("beforeDelete", {
          context,
          plugins: filePlugins,
          file
        });
        await storageOperations.delete({
          file
        });
        await (0, _lifecycleEvents.runLifecycleEvent)("afterDelete", {
          context,
          plugins: filePlugins,
          file
        });
      } catch (ex) {
        throw new _error.default(ex.message || "Could not delete a file.", ex.code || "DELETE_FILE_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          id,
          file
        }));
      }

      return true;
    },

    async createFilesInBatch(inputs) {
      if (!Array.isArray(inputs)) {
        throw new _error.default(`"data" must be an array.`, "CREATE_FILES_NON_ARRAY");
      }

      if (inputs.length === 0) {
        throw new _error.default(`"data" argument must contain at least one file.`, "CREATE_FILES_MIN_FILES");
      }

      if (inputs.length > BATCH_CREATE_MAX_FILES) {
        throw new _error.default(`"data" argument must not contain more than ${BATCH_CREATE_MAX_FILES} files.`, "CREATE_FILES_MAX_FILES");
      }

      await (0, _checkBasePermissions.default)(context, {
        rwd: "w"
      });
      const identity = context.security.getIdentity();
      const tenant = context.tenancy.getCurrentTenant();
      const createdBy = {
        id: identity.id,
        displayName: identity.displayName,
        type: identity.type
      };
      const files = inputs.map(input => {
        return (0, _objectSpread2.default)((0, _objectSpread2.default)({}, input), {}, {
          tags: Array.isArray(input.tags) ? input.tags : [],
          meta: (0, _objectSpread2.default)({
            private: false
          }, input.meta || {}),
          id: (0, _mdbid.default)(),
          tenant: tenant.id,
          createdOn: new Date().toISOString(),
          createdBy,
          locale: getLocaleCode(context),
          webinyVersion: context.WEBINY_VERSION
        });
      });

      try {
        await (0, _lifecycleEvents.runLifecycleEvent)("beforeBatchCreate", {
          context,
          plugins: filePlugins,
          data: files
        });
        const results = await storageOperations.createBatch({
          files
        });
        await (0, _lifecycleEvents.runLifecycleEvent)("afterBatchCreate", {
          context,
          plugins: filePlugins,
          data: files,
          files: results
        });
        return results;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not create a batch of files.", ex.code || "CREATE_FILES_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          files
        }));
      }
    },

    async listFiles(params = {}) {
      const permission = await (0, _checkBasePermissions.default)(context, {
        rwd: "r"
      });
      const {
        limit = 40,
        search = "",
        types = [],
        tags = [],
        ids = [],
        after = null,
        where: initialWhere,
        sort: initialSort
      } = params;
      const where = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, initialWhere), {}, {
        private: false,
        locale: getLocaleCode(context),
        tenant: context.tenancy.getCurrentTenant().id
      });
      /**
       * Always override the createdBy received from the user, if any.
       */

      if (permission.own === true) {
        const identity = context.security.getIdentity();
        where.createdBy = identity.id;
      }
      /**
       * We need to map the old GraphQL definition to the new one.
       * That GQL definition is marked as deprecated.
       */

      /**
       * To have standardized where objects across the applications, we transform the types into type_in.
       */


      if (Array.isArray(types) && types.length > 0 && !where.type_in) {
        where.type_in = types;
      }
      /**
       * We are assigning search to tag and name search.
       * This should be treated as OR condition in the storage operations.
       */


      if (search && !where.search) {
        where.search = search;
      }
      /**
       * Same as on types/type_in.
       */


      if (Array.isArray(tags) && tags.length > 0 && !where.tag_in) {
        where.tag_in = tags.map(tag => tag.toLowerCase());
      }
      /**
       * Same as on types/type_in.
       */


      if (Array.isArray(ids) && ids.length > 0 && !where.id_in) {
        where.id_in = ids;
      }

      const sort = Array.isArray(initialSort) && initialSort.length > 0 ? initialSort : ["id_DESC"];

      try {
        return await storageOperations.list({
          where,
          after,
          limit,
          sort
        });
      } catch (ex) {
        throw new _error.default(ex.message || "Could not list files by given parameters.", ex.code || "FILE_TAG_SEARCH_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          where,
          after,
          limit,
          sort
        }));
      }
    },

    async listTags({
      where: initialWhere,
      after,
      limit
    }) {
      await (0, _checkBasePermissions.default)(context);
      const where = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, initialWhere), {}, {
        tenant: context.tenancy.getCurrentTenant().id,
        locale: getLocaleCode(context)
      });
      const params = {
        where,
        limit: limit || 100000,
        after
      };

      try {
        const [tags] = await storageOperations.tags(params);

        if (Array.isArray(tags) === false) {
          return [];
        }
        /**
         * just to keep it standardized, sort by the tag ASC
         */


        return tags.sort();
      } catch (ex) {
        throw new _error.default(ex.message || "Could not search for tags.", ex.code || "FILE_TAG_SEARCH_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          params
        }));
      }
    }

  };
});
filesContextCrudPlugin.name = "FileManagerFilesCrud";
var _default = filesContextCrudPlugin;
exports.default = _default;