import { Plugin } from "@webiny/plugins";
import { File, FileManagerContext } from "../../types";
/**
 * Parameters for beforeCreate lifecycle.
 */
export interface BeforeCreateParams {
    context: FileManagerContext;
    /**
     * Data to be inserted into the storage.
     */
    data: File;
}
/**
 * Parameters for afterCreate lifecycle.
 */
export interface AfterCreateParams {
    context: FileManagerContext;
    /**
     * Data that was inserted into the storage.
     */
    data: File;
    /**
     * Result of the storage operations create method.
     * Possibly changed something on the "data".
     */
    file: File;
}
/**
 * Parameters for beforeUpdate lifecycle.
 */
export interface BeforeUpdateParams {
    context: FileManagerContext;
    /**
     * Original file from the storage.
     */
    original: File;
    /**
     * Data to be updated to the storage.
     */
    data: File;
}
/**
 * Parameters for afterUpdate lifecycle.
 */
export interface AfterUpdateParams {
    context: FileManagerContext;
    /**
     * Original file from the storage.
     */
    original: File;
    /**
     * Data that was updated in the storage.
     */
    data: File;
    /**
     * Result of the storage operations update method.
     * Possibly changed something on the "data".
     */
    file: File;
}
/**
 * Parameters for beforeBatchCreate lifecycle.
 */
export interface BeforeBatchCreateParams {
    context: FileManagerContext;
    /**
     * Files to be inserted into the storage.
     */
    data: File[];
}
/**
 * Parameters for afterBatchCreate lifecycle.
 */
export interface AfterBatchCreateParams {
    context: FileManagerContext;
    /**
     * Files that were inserted into the storage.
     */
    data: File[];
    /**
     * Results of the insert.
     */
    files: File[];
}
/**
 * Parameters for beforeDelete lifecycle.
 */
export interface BeforeDeleteParams {
    context: FileManagerContext;
    /**
     * File to be deleted from the storage.
     */
    file: File;
}
/**
 * Parameters for afterDelete lifecycle.
 */
export interface AfterDeleteParams {
    context: FileManagerContext;
    /**
     * File that was deleted from the storage.
     */
    file: File;
}
/**
 * Definition for the constructor parameters of the FilePlugin.
 *
 * @category FilePlugin
 */
export interface FilePluginParams {
    beforeCreate?: (params: BeforeCreateParams) => Promise<void>;
    afterCreate?: (params: AfterCreateParams) => Promise<void>;
    beforeUpdate?: (params: BeforeUpdateParams) => Promise<void>;
    afterUpdate?: (params: AfterUpdateParams) => Promise<void>;
    beforeBatchCreate?: (params: BeforeBatchCreateParams) => Promise<void>;
    afterBatchCreate?: (params: AfterBatchCreateParams) => Promise<void>;
    beforeDelete?: (params: BeforeDeleteParams) => Promise<void>;
    afterDelete?: (params: AfterDeleteParams) => Promise<void>;
}
export declare class FilePlugin extends Plugin {
    static readonly type: string;
    private readonly _params;
    constructor(params?: FilePluginParams);
    beforeCreate(params: BeforeCreateParams): Promise<void>;
    afterCreate(params: AfterCreateParams): Promise<void>;
    beforeUpdate(params: BeforeUpdateParams): Promise<void>;
    afterUpdate(params: AfterUpdateParams): Promise<void>;
    beforeBatchCreate(params: BeforeBatchCreateParams): Promise<void>;
    afterBatchCreate(params: AfterBatchCreateParams): Promise<void>;
    beforeDelete(params: BeforeDeleteParams): Promise<void>;
    afterDelete(params: AfterDeleteParams): Promise<void>;
    /**
     * Keep any here because it can be a number of params. Method is internal so no need to complicate the code.
     */
    private _execute;
}
